// Renderer2D Rect/Quad Shader

#type vertex
#version 450 core

layout(location = 0) in vec3 a_Position;
layout(location = 1) in vec3 a_Normal;
layout(location = 2) in vec4 a_Color;
layout(location = 3) in vec2 a_TexCoord;
layout(location = 4) in int a_GameObjectID;

layout(std140, binding = 0) uniform Camera
{
	mat4 u_ViewProjection;
};

struct VertexOutput
{
	vec4 Color;
	vec2 TexCoord;
};

layout (location = 0) out VertexOutput Output;
layout (location = 3) out flat int v_GameObjectID;

void main()
{
	Output.Color = a_Color;
	Output.TexCoord = a_TexCoord;
	v_GameObjectID = a_GameObjectID;

	gl_Position = u_ViewProjection * vec4(a_Position, 1.0);
}

#type fragment
#version 450 core

layout(location = 0) out vec4 o_Color;
layout(location = 1) out int o_GameObjectID;

struct VertexOutput
{
	vec4 Color;
	vec2 TexCoord;
};

layout (location = 0) in VertexOutput Input;
layout (location = 3) in flat int v_GameObjectID;

layout (binding = 0) uniform sampler2D u_Texture;

void main()
{
	vec4 texColor = Input.Color;

	texColor *= texture(u_Texture, Input.TexCoord);

	if (texColor.a == 0.0)
		discard;

	o_Color = texColor;
	o_GameObjectID = v_GameObjectID;
}